######################################################################
##
## Copyright 2011 Samuel Ytterbrink <neppord@gmail.com>
##
## Permission is hereby granted, free of charge, to any person
## obtaining a copy of this software and associated documentation
## files (the "Software"), to deal in the Software without
## restriction, including without limitation the rights to use,
## copy, modify, merge, publish, distribute, sublicense, and/or sell
## copies of the Software, and to permit persons to whom the
## Software is furnished to do so, subject to the following
## conditions:
##
## The above copyright notice and this permission notice shall be
## included in all copies or substantial portions of the Software.
##
## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
## OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
## NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
## HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
## WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
## FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
## OTHER DEALINGS IN THE SOFTWARE.
##
######################################################################

""" A formater module that keeps trac of indentation
"""
class Formater(object):
    """
    A very simple code formater that handles efficient concatenation and indentation of lines.
    """

    def __init__(self, indent_string="    "):
        self.__buffer = []
        self.__indentation = 0
        self.__indent_string = indent_string
        self.__indent_temp = ""
        self.__string_buffer = ""

    def dedent(self):
        """
        Subtracts one indentation level.
        """
        self.__indentation -= 1
        self.__indent_temp = self.__indent_string*self.__indentation

    def indent(self):
        """
        Adds one indentation level.
        """
        self.__indentation += 1
        self.__indent_temp = self.__indent_string*self.__indentation

    def write(self, text, indent=True, newline=True):
        """
        Writes the string text to the buffer with indentation and a newline if not specified otherwise.
        """
        if indent:
            self.__buffer.append(self.__indent_temp)
        self.__buffer.append(text)
        if newline:
            self.__buffer.append("\n")

    def read(self, size=None):
        """
        Returns a string representation of the buffer.
        """
        if size == None:
            text = self.__string_buffer + "".join(self.__buffer)
            self.__buffer = []
            self.__string_buffer = ""
            return text
        else:
            if len(self.__string_buffer) < size:
                self.__string_buffer += "".join(self.__buffer)
                self.__buffer = []
                if len(self.__string_buffer) < size:
                    text, self.__string_buffer = self.__string_buffer, ""
                    return text
                else:
                    text, self.__string_buffer = self.__string_buffer[:size], \
                        self.__string_buffer[size:]
                    return text
            else:
                text, self.__string_buffer = self.__string_buffer[:size], \
                    self.__string_buffer[size:]
                return text

